/*
 * VariableNumber.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util;


/**
 * An abstract base class for <code>Number</code>s that can change their value.
 * <br />
 * <br />
 * The mandate of a <code>VariableNumber</code> is to provide a number object
 * that can change value, thereby removing the need to instantiate a new object
 * each time the underlying value is changed.
 * <br />
 * However, it is important to respect the immutability of an object for
 * encapsulation purposes.
 * <br />
 * When a software module has created a variable number
 * (using its variable nature to alter its value during some processing
 * algorithm, say) it may want to expose that number externally. At this point
 * the number needs to lose its variability, to prevent other software modules
 * changing its value.
 * <br />
 * It is for this reason the variable number class provides the functionality
 * to secure the number, closing the object to any further change. Effectively
 * making it read-only.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.1 $
 * @since 1.1
 */
public abstract class VariableNumber extends Number
{
	
	/*
	 * Instance variables
	 */
	
	/** Flag set once the value has been fixed, effectively making the number
	 *  immutable. */
	private boolean secure = false;
	
	
	/*
	 * Methods
	 */
	
	/**
	 * Secure the value of this number.
	 * <br />
	 * Once this method has been called, the value is fixed. Calls to the
	 * <code>setValue</code> methods will cause an exception.
	 * <br />
	 * <br />
	 * @see #setValue(int)
	 * @see #setValue(long)
	 * @see #setValue(float)
	 * @see #setValue(double)
	 * @see #setValue(short)
	 * @see #setValue(byte)
	 */
	public void secure()
	{
		this.secure = true;
	}
	
	/**
	 * Check if the value of this number has been secured.
	 * <br />
	 * This method can be used to guard against exceptions being thrown by the
	 * <code>setValue</code> methods.
	 * <br />
	 * <br />
	 * @return <code>true</code> if the value of this number can no longer be
	 *         changed
	 * @see #setValue(int)
	 * @see #setValue(long)
	 * @see #setValue(float)
	 * @see #setValue(double)
	 * @see #setValue(short)
	 * @see #setValue(byte)
	 */
	public boolean isSecure()
	{
		return this.secure;
	}
	
	/**
	 * Check the secure flag, throwing an exception if the number is not in a
	 * variable state.
	 * <br />
	 * <br />
	 * @throws IllegalStateException if this number is no longer in a variable
	 *                               state
	 */
	protected void verifyNotSecure()
	{
		if (this.secure)
			throw new IllegalStateException( "Unable to change value of " + this.getClass().getName() + ". The value has been secured and is no longer open to change." );
	}
	
	/**
	 * Set the value of this number.
	 * <br />
	 * If the number is secure an exception will be thrown.
	 * <br />
	 * <br />
	 * @param value the new value for this number
	 * @throws IllegalStateException if this number is no longer in a variable
	 *                               state
	 * @see #secure
	 * @see #secure()
	 */
	public final void setValue( int value )
	{
		verifyNotSecure();
		setValueInternal( value );
	}
	
	/**
	 * Set the value of this number.
	 * <br />
	 * If the number is secure an exception will be thrown.
	 * <br />
	 * <br />
	 * @param value the new value for this number
	 * @throws IllegalStateException if this number is no longer in a variable
	 *                               state
	 * @see #secure
	 * @see #secure()
	 */
	public final void setValue( long value )
	{
		verifyNotSecure();
		setValueInternal( value );
	}
	
	/**
	 * Set the value of this number.
	 * <br />
	 * If the number is secure an exception will be thrown.
	 * <br />
	 * <br />
	 * @param value the new value for this number
	 * @throws IllegalStateException if this number is no longer in a variable
	 *                               state
	 * @see #secure
	 * @see #secure()
	 */
	public final void setValue( float value )
	{
		verifyNotSecure();
		setValueInternal( value );
	}
	
	/**
	 * Set the value of this number.
	 * <br />
	 * If the number is secure an exception will be thrown.
	 * <br />
	 * <br />
	 * @param value the new value for this number
	 * @throws IllegalStateException if this number is no longer in a variable
	 *                               state
	 * @see #secure
	 * @see #secure()
	 */
	public final void setValue( double value )
	{
		verifyNotSecure();
		setValueInternal( value );
	}
	
	/**
	 * Set the value of this number.
	 * <br />
	 * If the number is secure an exception will be thrown.
	 * <br />
	 * <br />
	 * @param value the new value for this number
	 * @throws IllegalStateException if this number is no longer in a variable
	 *                               state
	 * @see #secure
	 * @see #secure()
	 */
	public final void setValue( short value )
	{
		verifyNotSecure();
		setValueInternal( value );
	}
	
	/**
	 * Set the value of this number.
	 * <br />
	 * If the number is secure an exception will be thrown.
	 * <br />
	 * <br />
	 * @param value the new value for this number
	 * @throws IllegalStateException if this number is no longer in a variable
	 *                               state
	 * @see #secure
	 * @see #secure()
	 */
	public final void setValue( byte value )
	{
		verifyNotSecure();
		setValueInternal( value );
	}
	
	
	/*
	 * Abstract methods
	 */
	
	/**
	 * An internal set command.
	 * <br />
	 * This function will only be called if the number is still in a variable
	 * state.
	 * <br />
	 * <br />
	 * @param value the new value for this number
	 */
	protected abstract void setValueInternal( int value );
	
	/**
	 * An internal set command.
	 * <br />
	 * This function will only be called if the number is still in a variable
	 * state.
	 * <br />
	 * <br />
	 * @param value the new value for this number
	 */
	protected abstract void setValueInternal( long value );
	
	/**
	 * An internal set command.
	 * <br />
	 * This function will only be called if the number is still in a variable
	 * state.
	 * <br />
	 * <br />
	 * @param value the new value for this number
	 */
	protected abstract void setValueInternal( float value );
	
	/**
	 * An internal set command.
	 * <br />
	 * This function will only be called if the number is still in a variable
	 * state.
	 * <br />
	 * <br />
	 * @param value the new value for this number
	 */
	protected abstract void setValueInternal( double value );
	
	/**
	 * An internal set command.
	 * <br />
	 * This function will only be called if the number is still in a variable
	 * state.
	 * <br />
	 * <br />
	 * @param value the new value for this number
	 */
	protected void setValueInternal( short value )
	{
		setValueInternal( (int)value );
	}
	
	/**
	 * An internal set command.
	 * <br />
	 * This function will only be called if the number is still in a variable
	 * state.
	 * <br />
	 * <br />
	 * @param value the new value for this number
	 */
	protected void setValueInternal( byte value )
	{
		setValueInternal( (int)value );
	}
	
}


/*
 * Change History
 *
 * $Log: VariableNumber.java,v $
 * Revision 1.1  2006/10/15 21:20:29  Steve
 * Initial revision.
 *
 */