/*
 * RepaintThread
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util.gui;

import java.awt.Component;


/**
 * A <code>Thread</code> that causes a GUI component to refresh by periodically
 * calling the <code>repaint</code> method.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.3 $
 * @since 1.0
 */
public class RepaintThread extends Thread
{
	
	/*
	 * Instance variables
	 */
	
	/** The number of milli-seconds between each call to the repaint method. */
	protected long      _repaintPeriod;
	
	/** The UI component to be refreshed. */
	protected Component _component;
	
	/** A flag used to elegantly stop the thread. */
	protected boolean   _keepRepainting;
	
	
	/*
	 * Constructor(s)
	 */
	
	/**
	 * Create a thread that periodically calls repaint on a given component.
	 * <br />
	 * <br />
	 * @param component the GUI component to be periodically repainted
	 * @param period the time between each render, in milli-seconds
	 */
	public RepaintThread( Component component, long period )
	{
		_component = component;
		_repaintPeriod = period;
		_keepRepainting = true;
	}
	
	
	/*
	 * Implementation of abstract methods in Thread base-class
	 */
	
	public void run()
	{
		try
		{
			/* keep going until asked to stop */
			while (_keepRepainting)
			{
				Thread.sleep( _repaintPeriod );
				_component.repaint();
			}
		}
		catch (InterruptedException e)
		{
			/* not an error, just exit the thread */
		}
	}
	
	
	/*
	 * Methods
	 */
	
	/**
	 * Terminate this thead.
	 * <br />
	 * This happens asynchronously. This method may return without the thread
	 * actually having terminated but with the termination pending.
	 * <br />
	 * <br />
	 */
	public void doStop()
	{
		_keepRepainting = false;
	}
	
}


/*
 * Change History
 *
 * $Log: RepaintThread.java,v $
 * Revision 1.3  2006/09/06 21:07:54  Steve
 * Added legal disclaimer.
 *
 * Revision 1.2  2006/06/07 20:16:04  Steve
 * Brought into Bam library:
 *  - moved into lucidviews package
 *  - added CVS headers/footers
 *
 */
