/*
 * BaseImageObserver.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util.image;

import java.awt.image.ImageObserver;
import java.awt.Image;
import java.awt.Toolkit;

import java.util.Timer;
import java.util.TimerTask;


/**
 * A base class for <code>ImageObserver</code>s that observe (react to) a
 * specific image.
 * <br />
 * <br />
 * To prevent confusion between the <code>imageUpdate</code> method of the
 * <code>ImageObserver</code> interface and the <code>observedImageUpdate</code>
 * method of this class, the former has been made <code>final</code>.
 * <br />
 * Sub classes should implement the <code>observedImageUpdate</code> method in
 * exactly the same way they would implement the <code>imageUpdate</code>
 * method. When making the switch from <code>implements ImageObserver</code> to
 * <code>extends BaseImageObserver</code> you should only need to change the
 * name of your processing method from <code>imageUpdate</code> to
 * <code>observedImageUpdate</code>. 
 * <br />
 * <br />
 * This base class also implements functionality to make image observers
 * compatible with JRE 1.5.0_06 (see <code>keepObserverAlive</code>).
 * <br />
 * <br />
 * @see #keepObserverAlive
 * @see java.awt.image.ImageObserver
 * @see java.awt.Image
 * @author Stephen Battey
 * @version $Revision: 1.3 $
 * @since 1.0
 */
public abstract class BaseImageObserver implements ImageObserver
{
	
	/*
	 * Local constants
	 */
	
	/** The amount of time, in mill-seconds, an observer can go un */
	protected static final long FORCE_RESPAWN_DELAY = 1000;
	
	
	/*
	 * Instance variables
	 */
	
	/** The specific image being monitored by this observer. Leave as
	 *  <code>null</code> if not monitoring a particular image. */
	protected Image _image;
	
	/** A timer used to run a re-spawn task.
	 *  @see #keepObserverAlive() */ 
	private Timer _respawnTimer;
	
	/** The current re-spawn task running on the respawn timer.
	 *  @see #_respawnTimer  */
	private TimerTask _respawnTask;
	
	/** Flag indicating if this observer is still waiting for more data to load
	 *  into the image. */
	private boolean _needMoreData;
	

	
	/*
	 * Constructors
	 */
	
	/**
	 * Create an observer that is monitoring no specific image.
	 */
	public BaseImageObserver()
	{
		this( null );
	}
	
	/**
	 * Create an observer that is monitoring the specified image.
	 * <br />
	 * <br />
	 * @param image the image that is to be monitored
	 */
	public BaseImageObserver( Image image )
	{
		_image = image;
		_respawnTimer = new Timer();
		_respawnTask = null;
		_needMoreData = true;
		
		keepObserverAlive();
	}
	
	
	/*
	 * Abstract methods
	 */
	
	/**
     * This method is called when information about the image that is being
     * observed, and was previously requested using an asynchronous interface,
     * becomes available.
     * <br />
     * <br />
	 * This method should return true if further updates are needed or false if
	 * the required information has been acquired. The image which was being
     * tracked is passed in using the img argument. Various constants are
     * combined to form the infoflags argument which indicates what information
     * about the image is now available. The interpretation of the x, y, width,
     * and height arguments depends on the contents of the infoflags argument.
     * <br />
     * <br />
     * The <code>infoflags</code> argument should be the bitwise inclusive 
     * <b>OR</b> of the following flags: <code>WIDTH</code>, 
     * <code>HEIGHT</code>, <code>PROPERTIES</code>, <code>SOMEBITS</code>, 
     * <code>FRAMEBITS</code>, <code>ALLBITS</code>, <code>ERROR</code>, 
     * <code>ABORT</code>.
     * <br />
     * <br />
     * @param img the image being observed
     * @param infoflags the bitwise inclusive OR of the following 
     *                  flags:  <code>WIDTH</code>, <code>HEIGHT</code>, 
     *                  <code>PROPERTIES</code>, <code>SOMEBITS</code>,
     *                  <code>FRAMEBITS</code>, <code>ALLBITS</code>, 
     *                  <code>ERROR</code>, <code>ABORT</code> 
     * @param x the <i>x</i> coordinate
     * @param y the <i>y</i> coordinate
     * @param width the width
     * @param height the height
     * @return <code>false</code> if the infoflags indicate that the
     *         image is completely loaded; <code>true</code> otherwise
	 */
	protected abstract boolean observedImageUpdate( Image img, int infoflags, int x, int y, int width, int height );
	
	
	/*
	 * Implementation of ImageObserver interface
	 */
	
	public final boolean imageUpdate( Image img, int infoflags, int x, int y, int width, int height )
	{
		// the call to this method shows this observer is still alive
		// so cancel any timer task to respawn it
		cancelRespawn();
		
		// check this event is pertaining to the image we're observing
		if (   (_image == null)
		    || (_image == img)  )
		{
			_needMoreData = observedImageUpdate( img, infoflags, x, y, width, height );
		}
		
		if (_needMoreData)
			keepObserverAlive();
		
		//System.out.println( "BaseImageObserver::imageUpdate - needMoreData: " + _needMoreData );
		return _needMoreData;
	}
	
	/**
	 * Ensure this observer is kept alive until the image it is monitoring has
	 * been loaded completely.
	 * <br />
	 * This is done by scheduling a respawn task a second (or so) into the
	 * future. If this observer is forgotten then the task will timeout and call
	 * the respawn method of this observer, which will ensure this observer
	 * stays alive.
	 * <br />
	 * This technique was found to be necessary when running JImageTest in
	 * JRE 1.5.0_06. Other JREs (e.g. JRE 1.4) do not require this
	 * monitor-respawn functionality but it is included to ensure maximum
	 * compatibility.
	 * <br />
	 * This method should be called from the constructor and from the
	 * <code>imageUpdate</code> method whenever the observer is still waiting
	 * for more data.
	 */
	private void keepObserverAlive()
	{
		// cancel any old task that may be waiting
		cancelRespawn();
		
		_respawnTask = new RespawnTimerTask( this );
		_respawnTimer.schedule( _respawnTask, FORCE_RESPAWN_DELAY );
	}
	
	/**
	 * Cancel the respawn task, if such a task is active.
	 */
	private void cancelRespawn()
	{
		if (_respawnTask != null)
			_respawnTask.cancel();
		_respawnTask = null;
	}
	
	/**
	 * This method should be called when it is thought the observer has been
	 * 'lost' in the system.
	 */
	private void respawn()
	{
		// check the image using this observer,
		// which will re-associated this observer with the image
		Toolkit.getDefaultToolkit().checkImage( _image, -1, -1, this );
	}
	
	
	/*
	 * Helper classes
	 */
	
	/**
	 * A task that respawns an observer.
	 */
	protected static class RespawnTimerTask extends TimerTask
	{
		BaseImageObserver _observer;
		
		public RespawnTimerTask( BaseImageObserver observer )
		{
			_observer = observer;
		}
		
		public void run()
		{
			_observer.respawn();
		}
	}
	
}


/*
 * Change History
 *
 * $Log: BaseImageObserver.java,v $
 * Revision 1.3  2006/09/06 20:52:02  Steve
 * Added legal disclaimer.
 *
 * Revision 1.2  2006/09/02 16:53:23  Steve
 * Removed JavaDoc reference/link to test class (not available in JavaDoc).
 *
 * Revision 1.1  2006/09/02 16:36:15  Steve
 * Initial revision.
 *
 */
